% The simplest possible use of EM for estimating a dynamic system
% 
% Thomas Schn
% 22/4 - 2009

clear all;
close all;

randn('state',100);
rand('state',100);
addpath('H:\Matlab\unit');    % Add path to UNIT toolbox

opt.miter = 100;    % Max number of updates of estimate from initial guess
opt.LLdec = 1e-6;   % Min decrease of log-likelihood before termination
theta0 = 0.1;       % Initial guess for the a-parameter

%==========================================================
%===   Simulate the one dimensional state-space model   ===
%==========================================================
N = 500;            % Number of data
m.ss.A = 0.9;     m.ss.B = 0;
m.ss.C = 0.5;     m.ss.D = 0;
m.ss.Q = 0.1;     m.ss.S = 0;
m.ss.R = 0.1;

m.ss.X1 = 0;      m.ss.P1 = 0;  % Initial state (fully known)
x = zeros(1,N+1); y = zeros(1,N);
x(1) = m.ss.X1;
v = sqrt(m.ss.Q)*randn(1,N);    % Process noise sequence
e = sqrt(m.ss.R)*randn(1,N);    % Measurement noise sequence
for t=1:N
  x(t+1) = m.ss.A*x(t) + v(t);
  y(t)   = m.ss.C*x(t) + e(t);
end
z.y = y;

mEst = m;
mEst.ss.A = theta0;     % Initial parameter guess
%========================
%===   EM algorithm   ===
%========================
for k = 1:opt.miter
  % E step
  g     = ks(z,mEst,opt);       % Smoother
  LL(k) = -(0.5)*g.LL;          % Store the log-likelihood
  phi   = 0;   psi = 0;   sigma = 0;
  for t = 1:N
    phi   = phi + g.ss.xs(t+1)*g.ss.xs(t+1) + g.ss.Ps(1,1,t+1)^2;
    sigma = sigma + g.ss.xs(t).^2 + g.ss.Ps(1,1,t)^2;
    psi   = psi + g.ss.xs(t)*g.ss.xs(t+1) + g.ss.Ms(1,1,t);
  end;
  % M step
  a(k)      = psi/sigma;
  mEst.ss.A = a(k);

  % Compute the Q-function for the current estimate
  alpha = phi;
  beta  = 0;
  for t=1:N
    beta = beta + z.y(t)*g.ss.xs(t);
  end;
  aNow = [0:0.01:0.8 0.8:0.001:1 1.01:0.01:1.3];
  for i = 1:length(aNow)
    Q{k}(i) = -N*log(0.2*pi) - 5*sum(z.y.^2) - 5*alpha + 5*beta + 10*psi*aNow(i) - 5*sigma*(aNow(i)^2+1/4);
  end;

  if k>1          % Check termination condition
    if LL(k)-LL(k-1) < opt.LLdec
      break;
    end;
  end;
end;
a = [theta0 a];   % Add initial guess to the vector of estimates

% Compute the log-likelihood function
for i = 1:length(aNow)
  m.ss.A = aNow(i);
  g      = kf(z,m,opt);      % Kalman filter
  LL1(i) = -0.5*g.LL;        % UNIT toolbox likelihood
  Ltmp   = -(N/2)*log(2*pi);
  for t = 2:N
    CovInn   = g.ss.Ri(1,1,t)^2;
    Lpart1   = -(1/2)*((z.y(t) - m.ss.C*g.ss.xp(t))^2)/CovInn;
    Lpart2   = -(1/2)*log(det(CovInn));
    Ltmp     = Ltmp + Lpart1 + Lpart2;
  end;
  myLL(i) = Ltmp;
end;

%==============================
%===   Analyze the result   ===
%==============================
L = myLL;

figure2=figure(2);
axes2=axes('FontSize',12,'Parent',figure2);
set(figure2,'Position',[5 5 560 420]);     % [x y width height]
box('on');
hold('all');
plot1 = plot(aNow,L,'b');
plot2 = plot(aNow,Q{1},'r');
[mL, iL] = max(L);   [mQ, iQ] = max(Q{1});
iL = aNow(iL);       iQ = aNow(iQ);
plot3 = line([iL iL], [-6000 (mL + 40)]);
plot4 = line([iQ iQ], [-6000 (mQ + 40)]);
xlabel('Parameter a');
ylabel('Log-likelihood and Q function');
axis([0 1.3 -700 0])
set(plot1,'Color',[0.6 0.6 0.6],'LineWidth',2);
set(plot2,'Color',[0 0 0],'LineWidth',2);
set(plot3,'Color',[0 0 0],'LineWidth',1,'LineStyle','-');
set(plot4,'Color',[0 0 0],'LineWidth',1,'LineStyle','-');
legend('Log-likelihood', 'Q function')

figure3=figure(3);
axes3=axes('FontSize',12,'Parent',figure3);
box('on');
hold('all');
plot1 = plot(aNow,L);
plot2 = plot(aNow,Q{2});
[mQ, iQ] = max(Q{2});   iQ = aNow(iQ);
plot3 = line([iL iL], [-6000 (mL + 40)]);
plot4 = line([iQ iQ], [-6000 (mQ + 40)]);
xlabel('Parameter a');
ylabel('Log-likelihood and Q function');
axis([0 1.3 -700 0])
set(plot1,'Color',[0.6 0.6 0.6],'LineWidth',2);
set(plot2,'Color',[0 0 0],'LineWidth',2);
set(plot3,'Color',[0 0 0],'LineWidth',1,'LineStyle','-');
set(plot4,'Color',[0 0 0],'LineWidth',1,'LineStyle','-');
legend('Log-likelihood', 'Q function')

figure4 = figure(4);
axes4 = axes('FontSize',12,'Parent',figure4);
box('on');
hold('all');
plot1 = plot(aNow,L);
plot2 = plot(aNow,Q{3});
[mQ, iQ] = max(Q{3});   iQ = aNow(iQ);
plot3 = line([iL iL], [-6000 (mL + 40)]);
plot4 = line([iQ iQ], [-6000 (mQ + 40)]);
xlabel('Parameter a');
ylabel('Log-likelihood and Q function');
axis([0 1.3 -700 0])
set(plot1,'Color',[0.6 0.6 0.6],'LineWidth',2);
set(plot2,'Color',[0 0 0],'LineWidth',2);
set(plot3,'Color',[0 0 0],'LineWidth',1,'LineStyle','-');
set(plot4,'Color',[0 0 0],'LineWidth',1,'LineStyle','-');
legend('Log-likelihood', 'Q function')

figure5 = figure(5);
axes5 = axes('FontSize',12,'Parent',figure5);
box('on');
hold('all');
plot1 = plot(aNow,L);
plot2 = plot(aNow,Q{11});
[mQ, iQ] = max(Q{11});   iQ = aNow(iQ);
plot3 = line([iL iL], [-6000 (mL + 40)]);
plot4 = line([iQ iQ], [-6000 (mQ + 40)]);
xlabel('Parameter a');
ylabel('Log-likelihood and Q function');
axis([0 1.3 -700 0])
set(plot1,'Color',[0.6 0.6 0.6],'LineWidth',2);
set(plot2,'Color',[0 0 0],'LineWidth',2);
set(plot3,'Color',[0 0 0],'LineWidth',1,'LineStyle','-');
set(plot4,'Color',[0 0 0],'LineWidth',1,'LineStyle','-');
legend('Log-likelihood', 'Q function')

figure1 = figure(1);
axes1 = axes('FontSize',12,'Parent',figure1);
box('on');
hold('all');
plot1 = plot(a);
plot2 = line([0 length(a)],[0.9 0.9]);
axis([0 length(a) theta0-0.1 1.2])
xlabel('Iteration')
ylabel('Parameter a')
set(plot1,'Color',[0.6 0.6 0.6],'LineWidth',2);
set(plot2,'Color',[0 0 0],'LineWidth',2);
legend('Parameter estimates', 'True value')

figure;
plot (aNow,LL1,'b')
hold on
plot (aNow,myLL,'r')
